#!/usr/bin/perl -w

use strict;
use FindBin qw($RealDir);
use lib "$RealDir";
use File::Spec;
use File::Temp qw/ tempfile /;

use Data::Dumper;

our $opt_overwrite = 0;
our $opt_bypass = 0;
our $opt_verbose = 0;
our $progname = 'FIPS-ExtractSchedulesAux.pl';

our $xpathparser = undef;
our $domparsername = undef;
our $sub_readxmlfile = undef;
our $sub_createdocument = undef;
our $sub_xpathfindnodes = undef;
our $sub_xpathfindvalue = undef;

my $usage = <<EOM;
Usage:
  $progname [--overwrite] [--bypass] [--verbose] outputformat xmldir outputdir queryfiles...
  $progname [--overwrite] [--bypass] [--verbose] outputformat xmlfile outputdir queryfiles...
  $progname [--overwrite] [--bypass] [--verbose] outputformat xmlfile1,xmlfile2,... outputdir queryfiles...

This program should normally not be called directly, but through
fips-extract-sched.
EOM

my @savedARGV = @ARGV;
@ARGV = ();
while (@savedARGV) {
  my $arg = shift @savedARGV;
  if ($arg =~ /^--help$/) {
    print STDERR $usage;
    exit -1;
  } elsif ($arg =~ /^--overwrite$/) {
    $opt_overwrite++;
    next;
  } elsif ($arg =~ /^--bypass$/) {
    $opt_bypass++;
    next;
  } elsif ($arg =~ /^--verbose$/) {
    $opt_verbose++;
    next;
  } elsif ($arg =~ /^--/) {
    print STDERR "Unrecognized option $arg";
    print STDERR $usage;
    exit -1;
  }
  push @ARGV, $arg;
}

{
  eval { require XML::LibXML; };
  if (!$@) {
    $domparsername = 'XML::LibXML';
  } else {
    # no XML::LibXML, so try something else
    eval { require XML::XPath; };
    if ($@) {
      print STDERR "Can't find XML::LibXML, or XML::XPath\n";
      exit -1;
    }
    $domparsername = 'XML::XPath';
  }
  print STDERR "Using module '$domparsername':\n" if ($opt_verbose);
  if ($domparsername eq 'XML::XPath') {
    use XML::XPath;
    use XML::XPath::XMLParser;
    use XML::XPath::Node;
    use XML::XPath::Node qw/ :node_keys /;

    $sub_readxmlfile = sub {
      my ($filename,) = @_;
      my $parser = new XML::XPath::XMLParser;
      return $parser->parsefile($filename);
    };
    $sub_createdocument = sub {
      my ($rootname, ) = @_;
      my $doc = new XML::XPath::Node::Element;
      my $root = $doc->createElement($rootname);
      $doc->appendChild($root);
      return $doc;
    };
    $xpathparser = XML::XPath->new();
    $sub_xpathfindnodes = sub{
      my ($expr, $context) = @_;
      return $xpathparser->findnodes($expr, $context);
    };
    $sub_xpathfindvalue = sub{
      my ($expr, $context) = @_;
      my $retval = $context->findvalue($expr, $context);
      return undef if !defined($retval);
      return $retval->value();
    };

    package XML::XPath::NodeImpl;
    # XML::XPath::XMLParser uses the following structure for
    # XML nodes:
    #
    #  [
    #    node_parent,
    #    node_pos,
    #    ...type-specific data...
    #  ]
    #
    # Positions of various fields are imported through the :node_keys
    # import tag.
    use XML::XPath::Node;
    use XML::XPath::Node qw/ :node_keys /;
    sub cloneNode {
      my ($self, $deep) = @_;
      my $clone = bless([@$self], ref($self));
      $clone->[node_parent] = undef;
      if ($self->getNodeType() == ELEMENT_NODE) {
	if ($deep) {
	  my $children = $clone->[node_children];
	  my $attribs = $clone->[node_attribs];
	  my $namespaces = $clone->[node_namespaces];
	  if (defined($children)) {
	    $clone->[node_children] =
	      [
	       map {
		 $_->cloneNode(deep=>$deep);
	       } @$children
	      ];
	  }
	  if (defined($attribs)) {
	    $clone->[node_attribs] =
	      [
	       map {
		 $_->cloneNode(deep=>$deep);
	       } @$attribs
	      ];
	  }
	  if (defined($namespaces)) {
	    $clone->[node_namespaces] =
	      [
	       map {
		 $_->cloneNode(deep=>$deep);
	       } @$namespaces
	      ];
	  }
	} else {
	  $clone->[node_children] = [];
	  $clone->[node_attribs] = [];
	  $clone->[node_namespaces] = [];
	}
      }
      return $clone;
    }
    sub getNodeName {
      my ($self,) = @_;
      return $self->getName();
    }
    sub importNode {
      my ($self, $node, $deep) = @_;
      return $node->cloneNode(1);
    }
    sub createElement {
      my ($self, $tagname) = @_;
      return new XML::XPath::Node::Element($tagname);
    }
    sub createTextNode {
      my ($self, $content) = @_;
      return new XML::XPath::Node::Text($content);
    }
    sub getOwnerDocument {
      my ($self, ) = @_;
      my $parent = $self->getParentNode();
      if (defined($parent)) {
	return $parent->getOwnerDocument();
      } else {
	$self;
      }
    }
    sub setOwnerDocument {
      # no-op
    }
  } elsif ($domparsername eq 'XML::LibXML') {
    use XML::LibXML;
    $sub_readxmlfile = sub {
      my ($filename,) = @_;
      my $parser = XML::LibXML->new();
      return $parser->parse_file($filename);
    };
    $sub_createdocument = sub {
      my ($rootname, ) = @_;
      my $doc = XML::LibXML::Document->createDocument();
      my $root = $doc->createElement($rootname);
      $doc->setDocumentElement($root);
      return $doc;
    };
    $xpathparser = undef;
    $sub_xpathfindnodes = sub{
      my ($expr, $context) = @_;
      return $context->findnodes($expr);
    };
    $sub_xpathfindvalue = sub{
      my ($expr, $context) = @_;
      return $context->findvalue($expr);
    };

    package XML::LibXML::Node;
    sub getNodeType {
      my ($self, ) = @_;
      return $self->nodeType();
    };
    sub getNodeName {
      my ($self, ) = @_;
      return $self->nodeName();
    };
  }
}

sub clone_and_append_child {
  my ($doc, $parent_in, $child_in) = @_;
  my $newchild = $doc->importNode($child_in, 1);
  $parent_in->appendChild($newchild);
  return $newchild;
}

sub replace_onsetdur {
  # appending children is slow in XML::XPath, so avoid it when
  # possible
  my ($doc, $node, $newonset, $newdur) = @_;
  my @onsettextnodes = $sub_xpathfindnodes->('onset/text()', $node);
  my @durtextnodes = $sub_xpathfindnodes->('duration/text()', $node);
  if (@onsettextnodes > 1) {
    print STDERR "Too many onset nodes???\n";
    exit -1;
  }
  if (@durtextnodes > 1) {
    print STDERR "Too many duration nodes???\n";
    exit -1;
  }
  if (@onsettextnodes == 0) {
    my $newonsetnode = $doc->createElement('onset');
    $node->appendChild($newonsetnode);
    my $newonsettextnode = $doc->createTextNode("$newonset");
    $newonsetnode->appendChild($newonsettextnode);
    # do it in this order because it seems XML::XPath only increments
    # the global position number for each following node by a constant,
    # no matter if the appended node has children itself.  So, just
    # make sure we are appending from the top down.
  } else {
    $onsettextnodes[0]->setNodeValue("$newonset");
  }
  if (@durtextnodes == 0) {
    my $newdurnode = $doc->createElement('duration');
    $node->appendChild($newdurnode);
    my $newdurtextnode = $doc->createTextNode("$newonset");
    $newdurnode->appendChild($newdurtextnode);
    # do it in this order because it seems XML::XPath only increments
    # the global position number for each following node by a constant,
    # no matter what the content of the appended node is.  So, just
    # make sure we are appending from the top down.
  } else {
    $durtextnodes[0]->setNodeValue("$newdur");
  }
};

sub read_and_merge_events {
  my @xmlfiles = @_;
  my $mergedoc = $sub_createdocument->('events');
  my ($mergeeventselem, ) =
    $sub_xpathfindnodes->('events', $mergedoc);
  for my $xmlfile (@xmlfiles) {
    # find all <event> elements
    my $doc = $sub_readxmlfile->($xmlfile);
    # first find <events> elements
    my @eventselems = ();
    my @queue = ();
    for (my $child = $doc->getFirstChild();
	 defined($child);
	 $child = $child->getNextSibling()) {
      push @queue, $child;
    }
    while (@queue > 0) {
      my $node = shift @queue;
      if ($node->getNodeType() == ELEMENT_NODE &&
	  $node->getNodeName() eq 'events') {
	push @eventselems, $node;
      } else {
	for (my $child = $node->getFirstChild();
	     defined($child);
	     $child = $child->getNextSibling()) {
	  push @queue, $child;
	}
      }
    }
    # now find each child <event> element
    my @eventelems;
    for my $eventselem (@eventselems) {
      for (my $child = $eventselem->getFirstChild();
	   defined($child);
	   $child = $child->getNextSibling()) {
	if ($child->getNodeType() == ELEMENT_NODE &&
	    $child->getNodeName() eq 'event') {
	  push @eventelems, $child;
	}
      }
    }
    # for each <event> element, add them to mergedoc
    for my $eventelem (@eventelems) {
      clone_and_append_child($mergedoc, $mergeeventselem, $eventelem);
    }
#    $doc->dispose();
  }

  return ($mergedoc, $mergeeventselem);
}

#####################
# sort event document
sub sort_events {
  my ($doc_in, $eventselem_in) = @_;
  my @sortedeventlist = ();
  for (my $child = $eventselem_in->getFirstChild();
       defined($child);
       $child = $child->getNextSibling()) {
    my $onset = $sub_xpathfindvalue->('onset', $child);
    my $dur = $sub_xpathfindvalue->('duration', $child);
    next if !defined($onset);
    if ($onset =~ /^\s*$/) {
      next;
    }
    if (!defined($dur) || $dur =~ /^\s*$/) {
      $dur = 0;
    }
    push @sortedeventlist, [$onset, $dur, $child];
  }
  @sortedeventlist = sort {
    ($a->[0] <=> $b->[0]) || ($a->[1] <=> $b->[1])
  } @sortedeventlist;
  my $sortdoc = $sub_createdocument->('events');
  my ($sorteventselem, ) =
    $sub_xpathfindnodes->('events', $sortdoc);
  for my $event (@sortedeventlist) {
    $event->[2] = clone_and_append_child($sortdoc, $sorteventselem, $event->[2]);
  }
  return ($sortdoc, $sorteventselem, @sortedeventlist);
}

sub trans_events {
  my ($sortdoc, @sortedeventlist) = @_;
  print STDERR "Making transition document:\n" if ($opt_verbose);
  my $transdoc = $sub_createdocument->('events');
  my ($transeventselem, ) =
    $sub_xpathfindnodes->('events', $transdoc);
  my $epsilon = 0.0000001;	# nanosecond granularity
  while (@sortedeventlist > 1) {
    my $eventA = $sortedeventlist[0];
    my $eventB = $sortedeventlist[1];
    my ($onsetA, $durA, $nodeA) = @$eventA;
    my ($onsetB, $durB, $nodeB) = @$eventB;
    print STDERR "A: [$onsetA, $durA)\tB: [$onsetB, $durB]\n" if ($opt_verbose);
    if ($onsetA != $onsetB && $onsetA + $durA <= $onsetB) {
      print STDERR " Retiring A (no overlap)\n" if ($opt_verbose);
      # no interval overlap, so eventA is good to go
      clone_and_append_child($transdoc, $transeventselem, $nodeA);
      shift @sortedeventlist;
      next;
    }
    if (abs($onsetA - $onsetB) < $epsilon && abs($durA - $durB) < $epsilon) {
      # intervals A and B are equal
      # need to merge B into A and delete B
      print STDERR " Merging B into A, deleting B\n" if ($opt_verbose);
      my $newnodeA = $nodeA->cloneNode(1);
      my @valuenodes = $sub_xpathfindnodes->('value', $nodeB);
      for my $valuenode (@valuenodes) {
	clone_and_append_child($sortdoc, $newnodeA, $valuenode);
      }
      $sortedeventlist[0]->[2] = $newnodeA;
      # delete interval B
      $sortedeventlist[1] = $sortedeventlist[0];
      shift @sortedeventlist;
      next;
    }
    # if we get here, A and B overlap, and starting point of A
    # is no greater than starting point of B, because they are
    # sorted
    if (abs($durA) < $epsilon) {
      # Interval A [x,x], is a single point and starts at the
      # same time as interval B [x,y], by virtue of ordering
      # constraints.  Merge B into A, but keep B.
      print STDERR " Merging B into A, keeping B\n" if ($opt_verbose);
      my $newnodeA = $nodeA->cloneNode(1);
      my @valuenodes = $sub_xpathfindnodes->('value', $nodeB);
      for my $valuenode (@valuenodes) {
	clone_and_append_child($sortdoc, $newnodeA, $valuenode);
      }
      # eventA is finished
      print STDERR " Retiring A\n" if ($opt_verbose);
      clone_and_append_child($transdoc, $transeventselem, $newnodeA);
      shift @sortedeventlist;
      next;
    }
    if ($onsetA < $onsetB) {
      # Interval A [w,x) overlaps and starts before
      # interval B [y,z):
      #     |-----A-----|
      #  <--w-----y-----x-----z-->
      #           |-----B-----|
      # or
      #     |--------A--------|
      #  <--w-----y-----z-----x-->
      #           |--B--|
      # Split interval A into two fragments C [a1,b1) and D [b1,a2):
      #     |--C--|--D--|
      #  <--w-----y-----x-----z-->
      #           |-----B-----|
      # or
      #     |--C--|-----D-----|
      #  <--w-----y-----z-----x-->
      #           |--B--|
      #
      my $nodeC = $nodeA->cloneNode(1);
      my $nodeD = $nodeA->cloneNode(1);
      my $onsetC = $onsetA;
      my $durC = $onsetB - $onsetA;
      my $onsetD = $onsetB;
      my $durD = ($onsetA + $durA) - $onsetB;
      replace_onsetdur($sortdoc, $nodeC, $onsetC, $durC);
      replace_onsetdur($sortdoc, $nodeD, $onsetD, $durD);
      print STDERR " Splitting A into two intervals:\n" if ($opt_verbose);
      print STDERR "  C: [$onsetC, $durC]\tD: [$onsetD, $durD]\n" if ($opt_verbose);
      print STDERR " Retiring C\n" if ($opt_verbose);
      # nodeA now has interval C, and nodeD has interval D
      # C is guaranteed to not overlap any other intervals,
      # so we can get rid of eventA
      clone_and_append_child($transdoc, $transeventselem, $nodeC);
      shift @sortedeventlist;
      # add new interval D to sorted event list
      my $putbefore = -1;
      for my $ind (0..$#sortedeventlist) {
	my ($tmponset, $tmpdur, $tmpevent) = @{$sortedeventlist[$ind]};
	if ($tmponset > $onsetD || ($tmponset == $onsetD && $tmpdur > $durD)) {
	  $putbefore = $ind;
	  last;
	}
      }
      my $neweventref = [$onsetD, $durD, $nodeD];
      if ($putbefore == -1) {
	push @sortedeventlist, $neweventref;
      } else {
	splice(@sortedeventlist, $putbefore, 0, $neweventref);
      }
      next;
    }
    if (abs($onsetA - $onsetB) < $epsilon && $durA < $durB) {
      # Interval A [x,y) starts at the same time, but
      # ends before interval B [x,z):
      #     |--A--|
      #  <--x-----y-----z-->
      #     |-----B-----|
      # Split interval B into two fragments C [x,y) and D [y,z).
      #     |--A--|
      #  <--x-----y-----z-->
      #     |--C--|--D--|
      #
      my $nodeC = $nodeB->cloneNode(1);
      my $nodeD = $nodeB->cloneNode(1);
      my $onsetC = $onsetB;
      my $durC = $durA;
      my $onsetD = $onsetB + $durA;
      my $durD = $durB - $durA;
      replace_onsetdur($sortdoc, $nodeC, $onsetC, $durC);
      replace_onsetdur($sortdoc, $nodeD, $onsetD, $durD);
      print STDERR " Splitting B into two intervals:\n" if ($opt_verbose);
      print STDERR "  C: [$onsetC, $durC]\tD: [$onsetD, $durD]\n" if ($opt_verbose);
      # nodeB now has interval C, and nodeD has interval D.
      # new intervals C and D need to be added to sorted event list.
      # interval C can just take B's old place (right after A, since
      # they share the same onset and duration)
      $eventB->[0] = $onsetC;
      $eventB->[1] = $durC;
      $eventB->[2] = $nodeC;
      # add new interval D to sorted event list
      my $putbefore = -1;
      for my $ind (0..$#sortedeventlist) {
	my ($tmponset, $tmpdur, $tmpevent) = @{$sortedeventlist[$ind]};
	if ($tmponset > $onsetD || ($tmponset == $onsetD && $tmpdur > $durD)) {
	  $putbefore = $ind;
	  last;
	}
      }
      my $neweventref = [$onsetD, $durD, $nodeD];
      if ($putbefore == -1) {
	push @sortedeventlist, $neweventref;
      } else {
	splice(@sortedeventlist, $putbefore, 0, $neweventref);
      }
      next;
    }
    print STDERR "$progname: Internal error: events are not sorted correctly?\n";
  }
  # there should be at most one element left in sortedeventlist
  map {
    my $node = $_->[2];
    clone_and_append_child($transdoc, $transeventselem, $node);
  } @sortedeventlist;

  return ($transdoc, $transeventselem);
}

##############
# MAIN PROGRAM
##############

if (@ARGV < 4) {
  print STDERR "$progname: ERROR: wrong number of arguments.\n";
  print STDERR $usage;
  exit -1;
}

my $outputformat = shift @ARGV;
my $xmldir = shift @ARGV;
my @xmlfiles = ();
my $outputdir = shift @ARGV;
my @queryfiles = @ARGV;
@ARGV = ();

if ($outputformat ne 'fsl') {
  print STDERR "$progname: ERROR: Unrecognized output format '$outputformat'\n";
  exit -1;
}

if (-d $xmldir) {
  # find all XML events files
  my $xmlglob = File::Spec->catfile($xmldir, '*.xml');
  @xmlfiles = glob $xmlglob;

  if (@xmlfiles == 0) {
    if ($opt_bypass) {
      print STDERR "$progname: WARNING: No files matched '$xmlglob'.\n";
      exit 0;
    } else {
      print STDERR "$progname: ERROR: No files matched '$xmlglob'.\n";
      exit -1;
    }
  }
} else {
  @xmlfiles = split(/,/, $xmldir);
}
for my $xmlfile (@xmlfiles) {
  if (! -f $xmlfile) {
    if ($opt_bypass) {
      print STDERR "$progname: WARNING: $xmlfile does not exist (or is not a file).\n";
      exit 0;
    } else {
      print STDERR "$progname: ERROR: $xmlfile does not exist (or is not a file).\n";
      exit -1;
    }
  }
}

if (! -d $outputdir) {
  print STDERR "$progname: ERROR: $outputdir does not exist.\n";
  exit -1;
}

##########################
# read and parse queryfile
my @querylanguages = ();
my @querylabels = ();
my @queries = ();
my @queryfilters = ();
my @queryepochexcludes = ();
my @epochsecsbefores = ();
my @epochsecsafters = ();
for my $queryfile (@queryfiles) {
  my %queryopts = ();
  open(QFH, $queryfile) || die "$progname: ERROR opening '$queryfile': $!\n";
  while (<QFH>) {
    chomp;
    my $outline = '';
    # remove comments from input line
    while (length($_) > 0) {
      # the following patterns are followed by comments with strange
      # characters to overcome the limitations of emacs' cperl-mode
      # syntax highlighting.
      s/^(\#.*)//;
      s/^([^'"]+)//  && ($outline .= $1); #']);
      s/^('[^']*')// && ($outline .= $1); #';
      s/^("[^"]*")// && ($outline .= $1); #";
    }
    $_ = $outline;
    # remove initial and trailing whitespace from input line
    s/^\s+//;
    s/\s+$//;
    # option name is initial string of non-whitespace characters
    my $optname = undef;
    s/^(\S+)\s*// && ($optname = $1);
    next if !defined($optname);
    # option arg may be quoted arbitrarily, so "unquote" it
    my $optarg = '';
    while (length($_) > 0) {
      s/^([^'"]+)//  && ($optarg .= $1); #']);
      s/^\'([^']*)\'// && ($optarg .= $1); #']);
      s/^\"([^"]*)\"// && ($optarg .= $1); #"]);
    }
    push @{$queryopts{$optname}}, $optarg;
  }
  close(QFH);

  if (!exists($queryopts{'querylanguage'})) {
    print STDERR "$progname: ERROR: query file '$queryfile' missing 'querylanguage' line\n";
    exit -1;
  }
  if (!exists($queryopts{'querylabel'})) {
    print STDERR "$progname: ERROR: query file '$queryfile' missing 'querylabel' line(s)\n";
    exit -1;
  }
  if (!exists($queryopts{'query'})) {
    print STDERR "$progname: ERROR: query file '$queryfile' missing 'query' line(s)\n";
    exit -1;
  }
  if (!exists($queryopts{'queryfilter'})) {
    $queryopts{'queryfilter'} = [];
  }
  if (!exists($queryopts{'queryepochexclude'})) {
    $queryopts{'queryepochexclude'} = [];
  }

  my $numquerylabels = @{$queryopts{'querylabel'}};
  my $numqueries = @{$queryopts{'query'}};
  my $numqueryfilters = @{$queryopts{'queryfilter'}};
  my $numqueryepochexcludes = @{$queryopts{'queryepochexclude'}};

  if ($numquerylabels != $numqueries) {
    print STDERR "$progname: ERROR: Number of 'querylabel' ($numquerylabels) and 'query' ($numqueries) lines in query file '$queryfile' is not the same.\n";
    exit -1;
  }
  if ($numqueryfilters > 0 &&
      $numqueryfilters != $numqueries) {
    print STDERR "$progname: ERROR: Number of 'queryfilters' ($numqueryfilters) and 'query' ($numqueries) lines in query file '$queryfile' is not the same.\n";
    exit -1;
  }
  if ($numqueryepochexcludes > 0 &&
      $numqueryepochexcludes != $numqueries) {
    print STDERR "$progname: ERROR: Number of 'queryepochexcludes' ($numqueryepochexcludes) and 'query' ($numqueries) lines in query file '$queryfile' is not the same.\n";
    exit -1;
  }

  if ($numqueryepochexcludes > 0) {
    if (!exists($queryopts{'secsbefore'}) ||
	!exists($queryopts{'secsafter'})) {
      print STDERR "$progname: ERROR: 'secsbefore' and 'secsafter' lines are required if using 'queryepochexclude' in query file '$queryfile'.\n";
      exit -1;
    }
  }
  # make the arrays equal size (undefs if missing)
  $#{$queryopts{'querylabel'}} = $numqueries-1;
  $#{$queryopts{'queryfilter'}} = $numqueries-1;
  $#{$queryopts{'queryepochexclude'}} = $numqueries-1;
  # replicate the querylanguage for each query
  while (@{$queryopts{'querylanguage'}} < $numqueries) {
    push @{$queryopts{'querylanguage'}}, $queryopts{'querylanguage'}->[0];
  }
  # replicate the secsbefore for each query
  if ($numqueries > 0 && !exists($queryopts{'secsbefore'})) {
    $queryopts{'secsbefore'} = [undef];
  }
  while (@{$queryopts{'secsbefore'}} < $numqueries) {
    push @{$queryopts{'secsbefore'}}, $queryopts{'secsbefore'}->[0];
  }
  # replicate the secsafter for each query
  if ($numqueries > 0 && !exists($queryopts{'secsafter'})) {
    $queryopts{'secsafter'} = [undef];
  }
  while (@{$queryopts{'secsafter'}} < $numqueries) {
    push @{$queryopts{'secsafter'}}, $queryopts{'secsafter'}->[0];
  }

  # now add them to global lists
  push @querylanguages, @{$queryopts{'querylanguage'}};
  push @querylabels, @{$queryopts{'querylabel'}};
  push @queries, @{$queryopts{'query'}};
  push @queryfilters, @{$queryopts{'queryfilter'}};
  push @queryepochexcludes, @{$queryopts{'queryepochexclude'}};
  push @epochsecsbefores, @{$queryopts{'secsbefore'}};
  push @epochsecsafters, @{$queryopts{'secsafter'}};
}

if (!$opt_overwrite) {
  my $needtowrite = 0;
  for my $querylabel (@querylabels) {
    my $outputfile = undef;
    if ($outputformat eq 'fsl') {
      $outputfile = "${querylabel}.stf";
    }
    my $fulloutputfile = File::Spec->catfile($outputdir, $outputfile);
    if (-e $fulloutputfile) {
      if (!$opt_bypass) {
	print STDERR "$progname: ERROR: output file '$outputfile' exists.\n";
	print STDERR "  Delete it or run with --overwrite or --bypass\n";
	exit -1;
      }
    } else {
      $needtowrite = 1;
    }
  }
  if ($opt_bypass && $needtowrite == 0) {
    print STDOUT "Skipping directory (output files already exist).\n";
    exit 0;
  }
}

###################################
# Convert XCEDE queries into XPath
for my $querynum (0..$#queries) {
  if ($querylanguages[$querynum] eq 'event' ||
      $querylanguages[$querynum] eq 'XCEDE') {
    for my $query ($queries[$querynum], $queryfilters[$querynum], $queryepochexcludes[$querynum]) {
      next if !defined($query);
      open(CONVFH, '-|', 'xcede-query2xpath', $query) ||
	die "Error running xcede-query2xpath: $!\n";
      my $result = <CONVFH>;
      chomp $result;
      close CONVFH;
      print STDERR "XCEDE query '$query' converted to XPath query '$result'\n" if ($opt_verbose);
      $query = $result;
    }
  }
}

####################################################
# read all XML events files, merge them together, and sort them
print STDERR "Reading/merging events...\n" if ($opt_verbose);
my ($mergedoc, $mergeeventselem) = read_and_merge_events(@xmlfiles);
print STDERR "Sorting events...\n" if ($opt_verbose);
my ($sortdoc, $sorteventselem, @sortedeventlist) =
  sort_events($mergedoc, $mergeeventselem);

#################################################################
# If needed, make a "transition" document, which has an event for
# every time interval in the merged event list that with a unique
# set of simultaneous events (i.e. basically a timeline).
my $transdoc = undef;
my $transeventselem = undef;
###XXX
if (grep { defined($_) } (@queryfilters, @queryepochexcludes)) {
  print STDERR "Making transition event list...\n" if ($opt_verbose);
  ($transdoc, $transeventselem) = trans_events($sortdoc, @sortedeventlist);
}

############################################################
# We now have a document with all original events sorted,
# and a "transition" document marking all event transitions.
# We can now perform the queries.
print STDERR "Querying...\n" if ($opt_verbose);
my $queryprefix = "event";
for my $querynum (0..$#queries) {
  my $querylabel = $querylabels[$querynum];
  my $query = $queries[$querynum];
  my $queryfilter = undef;
  my $queryepochexclude = undef;
  $queryfilter = $queryfilters[$querynum];
  $queryepochexclude = $queryepochexcludes[$querynum];
  my @matchednodes = $sub_xpathfindnodes->("${queryprefix}\[${query}]", $sorteventselem);
  my @matchedonsetdurs =
    map { [
	   $sub_xpathfindvalue->('onset', $_) || 0,
	   $sub_xpathfindvalue->('duration', $_) || 0
	  ] } @matchednodes;
  # this shouldn't be necessary, but XML::XPath appending is a little
  # weird, resulting in unsorted results
  @matchedonsetdurs =
    sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @matchedonsetdurs;
  if (defined($queryfilter)) {
    my @filternodes = $sub_xpathfindnodes->("${queryprefix}\[${queryfilter}]", $transeventselem);
    my @filteronsetdurs =
      map { [
	     $sub_xpathfindvalue->('onset', $_) || 0,
	     $sub_xpathfindvalue->('duration', $_) || 0
	    ] } @filternodes;
    @filteronsetdurs =
      sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @filteronsetdurs;
    @matchedonsetdurs =
      grep {
	my $monset = $_->[0];
	grep {
	  my ($fonset, $fdur) = @$_;
	  ($fonset <= $monset && $monset < $fonset + $fdur);
	} @filteronsetdurs;
      } @matchedonsetdurs;
  }
  if (defined($queryepochexclude)) {
    my $epochsecsbefore = $epochsecsbefores[$querynum];
    my $epochsecsafter = $epochsecsafters[$querynum];
    my @excludenodes = $sub_xpathfindnodes->("${queryprefix}\[${queryepochexclude}]", $transeventselem);
    my @excludeonsetdurs =
      map { [
	     $sub_xpathfindvalue->('onset', $_) || 0,
	     $sub_xpathfindvalue->('duration', $_) || 0
	    ] } @excludenodes;
    @excludeonsetdurs =
      sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @excludeonsetdurs;
    @matchedonsetdurs =
      grep {
	my $monset = $_->[0];
	!scalar(grep {
	  my ($exonset, $exdur) = @$_;
	  (($exonset - $epochsecsafter < $monset) &&
	   ($monset < $exonset + $exdur + $epochsecsbefore));
	} @excludeonsetdurs);
      } @matchedonsetdurs;
  }

  my $outputfile = undef;
  if ($outputformat eq 'fsl') {
    $outputfile = "${querylabel}.stf";
  }
  my $fulloutputfile = File::Spec->catfile($outputdir, $outputfile);
;
  if ($opt_bypass && -e $fulloutputfile) {
    print STDOUT "File exists, skipping $outputfile.\n";
  } else {
    print STDOUT "Writing $outputfile...\n";
    open(OFH, '>', $fulloutputfile) ||
      die "$progname: Error opening '$fulloutputfile' for writing: $!\n";
    if ($outputformat eq 'fsl') {
      print OFH join("\n", map { join("\t", @$_) . "\t1" } @matchedonsetdurs), "\n";
    }
    close OFH;
  }
}

#$sortdoc->dispose();
#if (defined($transdoc)) {
#  $transdoc->dispose();
#}
