#!/usr/bin/perl
# FIPS-SetDatarec.pl
# $Id: FIPS-SetDatarec.pl,v 1.1 2006/05/31 18:59:12 rnotestine Exp $
################################################################################
# Dependencies:
#               XML::XPath - SRB@/home/dkeator@uci-bic/NonHumanPhantomScript/XML
#
# Authors:      David Keator (dbkeator@uci.edu)
#               Syam Gadde (gadde@biac.duke.edu)
#               Doug Greve (greve@nmr.mgh.harvard.edu)
#               Randy Notestine (RNotestine@UCSD.edu)
#
# Revision History:
#   2006/05/22  Code taken from LocalHumanUpload/RemoveSeries.pl and
#               fips/FIPS-SetAnalysisLevel.pl
#
# Notes:        This program does NOT currently support multiple filename,
#               fileoffset, or filerecordsize nodes.  However, multiple
#               fileoffset or filerecordsize values (within a single node) can
#               be set by enclosing the space separated list of values in quotes.
################################################################################

use FindBin qw($RealDir);
use lib "$RealDir";
require 'FIPS-XMLFuncs.pl';

# Check number of arguments
my $nargs = $#ARGV + 1;
if($nargs < 4) {
  print "\nUsage: FIPS-SetDatarec.pl xml_in xml_out element1 element1_value [ element2 element2_value [ ... ] ]\n\n";
  print "  Sets (modify or create) one or more of the following datarec elements:\n";
  print "    \"byteorder\", \"elementtype\", \"filename\", \"fileoffset\", and/or \"filerecordsize\"\n\n";
  print "  Warning: Multiple filename/fileoffset/filerecordsize nodes will be truncated\n";
  print "    if any of these three elements are set.\n\n";
  exit(1);
}

my $xml_in      = shift @ARGV;
my $xml_out     = shift @ARGV;

if(! -e $xml_in ) {
  print "ERROR: xml input file \"$xml_in\" does not exist.\n";
  exit(1);
}

# Standard Stuff to read in an XML document
my $parser = new XML::DOM::Parser;
my $doc = $parser->parsefile("$xml_in");
my $root_node = $doc->getDocumentElement();

my $datarec_xpath = "serieslevel/datarec";
my $datarec_node = findNode($doc, $datarec_xpath);
my $truncate_file_nodes = 0;  # Do not truncate multiple file* nodes

# Determine if any filename/fileoffset/filerecordsize nodes are being set
for ($arg_i = 0; $arg_i < $#ARGV; $arg_i += 2) {
  if ($ARGV[$arg_i] eq "filename" || $ARGV[$arg_i] eq "fileoffset" || $ARGV[$arg_i] eq "filerecordsize") {
    $truncate_file_nodes = 1;
  }
}

# Truncate (remove multiple occurances of) filename/fileoffset/filerecordsize nodes, if required
if ($truncate_file_nodes) {
  my $file_count = 0;
  foreach my $child ($datarec_node->getChildNodes()) {
    if ($child->getNodeName =~ m/filename/) {
      $file_count++;
      if ($file_count > 1) {
        $datarec_node->removeChild($child);
      }
    } elsif (($child->getNodeName =~ m/fileoffset/ || $child->getNodeName =~ m/filerecordsize/) && $file_count > 1) {
      $datarec_node->removeChild($child);
    }
  }
}

# Loop over specified datarec elements
while ($#ARGV > 0) {
  $element_name  = shift @ARGV;
  $element_value = shift @ARGV;
  $element_unmatched = 1;

  # Validate specified element_name and element_value
  if ($element_name eq "byteorder") {
    if ("lsbfirst msbfirst" !~ m/$element_value/) {
      print "Error: Unsupported byteorder \"$element_value\" specified\n";
      exit (1);
    }
  } elsif ($element_name eq "elementtype") {
    if ("int8 uint8 int16 uint16 int32 uint32 int64 uint64 float32 float64 ascii" !~ m/$element_value/) {
      print "Error: Unsupported elementtype \"$element_value\" specified\n";
      exit (1);
    }
  } elsif ($element_name eq "filename") {
    # Remove .gz extension if present
    $element_value =~ s/.gz$//;
  } elsif ($element_name eq "fileoffset") {
    # Should add a check to ensure that fileoffset and filerecordsize are consistent
  } elsif ($element_name eq "filerecordsize") {
    # Should add a check to ensure that fileoffset and filerecordsize are consistent
  } else {
    print "Error: Unsupported datarec element \"$element_name\" specified\n";
    exit (1);
  }

  print "  Setting datarec element \"$element_name\" to \"$element_value\"\n";

  # Loop over child nodes
  foreach my $child ($datarec_node->getChildNodes()) {

    # Look for match
    if ($child->getNodeName =~ m/$element_name/) {

      # Set element_value, set match flag, and advance to next ARGV pair (break out of foreach loop)
      setValue($doc, "$datarec_xpath/$element_name", $element_value);
      $element_unmatched = 0;
      last;
    }
  }

  # If element wasn't matched, append new node
  if ($element_unmatched) {
    appendNode($doc, $datarec_node, $element_name, $element_value);
  }
}

# removeChild typically leaves near-blank lines (spaces/tabs with newlines) where children were removed,
# so remove these near-blank lines before writing new XML file
# $doc->printToFile("$xml_out");
my $string = $doc->toString;
$string =~ s/^[ \t]+\n//gm;
open(OUTXML, ">$xml_out");
print OUTXML $string;
close(OUTXML);

# Avoid memory leaks - cleanup circular references
$doc->dispose;

exit(0);
