function [qaregstruct] = LoadQARegSum(regsumfile)
% qaregstruct = LoadQARegSum(regsumfile)
%
% Loads in the file created with fips-qa-sum. Eg:
%   fips-qa-sum --reg reg.dat --flac fb3-wm-sm5
%
% qastruct has the following members:
%  qatable 
%  birnids - N-by-1 string array of birnids 
%  visitdates - N-by-1 string array of  YYYY-MM-DD
%  visitdatenums - N-by-1 array of the visit date 
%     expressed as the number of days since Jan 1 2007.
%  acqsites - N-by-1 string array of acquisition sites
%  acqsitenos - N-by-1 array of acquisition sites numbers
%  visitnos - N-by-1 vector of visit numbers
%  runnos - N-by-1 vector of run numbers
%  measnames - 2-by-M string array of the 2 Reg measures
% 
% qaregtable is N-by-13
% 1. determinant
% 2. M11
% 3. M12
% 4. M13
% 5. M14
% 6. M21
% ...
%
% $Id: LoadQARegSum.m,v 1.1 2007/10/09 21:10:53 dgreve Exp $

qaregstruct = [];

if(nargin ~= 1)
  fprintf('qaregstruct = LoadQARegSum(regsumfile)\n');
  return
end

measnames = '';
measnames = strvcat(measnames,'Reg Abs');
measnames = strvcat(measnames,'Reg Rel');

fp = fopen(regsumfile,'r');
if(fp == -1)
  fprintf('ERROR: could not open %s\n',regsumfile);
  return;
end

birnids = [];
visitdates = [];
acqsites = [];
acqsitenos = [];
visitnos = [];
runnos = [];
qatable = [];
vdatenums = [];

% Serial date number of Jan 1 2007 to use as a baseline
vdatenum0 = datenum(2007,1,1);

tic;
nrows = 0;
while(1)
  nrows = nrows + 1;

  fscanf(fp,'%s',1); % past nth
  [bid   count] = fscanf(fp,'%s',1);
  if(count ~= 1) break; end
  [vdate count]   = fscanf(fp,'%s',1);
  [acqsite count] = fscanf(fp,'%s',1);
  acqsiteno = sscanf(acqsite,'%d');
  [vno count]     = fscanf(fp,'%d',1);
  [rno count]     = fscanf(fp,'%d',1);
  [qaline count]  = fscanf(fp,'%f',13);
  
  birnids = strvcat(birnids,bid);
  visitdates = strvcat(visitdates,vdate);
  acqsites   = strvcat(acqsites,acqsite);
  acqsitenos = [acqsitenos; acqsiteno];
  visitnos = [visitnos; vno];
  runnos = [runnos; rno];
  
  qatable = [qatable; qaline'];

  % This computes the serial date number for the visit
  % relative to Jan 1 2007
  ind = find(vdate == '-');
  vdate(ind) = ' ';
  ymd = sscanf(vdate,'%d %d %d');
  vdatenum = datenum(ymd(1),ymd(2),ymd(3)) - vdatenum0;
  vdatenums = [vdatenums; vdatenum];
  
end
%fprintf('%4d %g\n',nrows,toc);

fclose(fp);

% Sort by Acq Date
[a ii] = sort(vdatenums);

qaregstruct.qatable = qatable(ii,:);
qaregstruct.birnids = birnids(ii,:);
qaregstruct.visitdates = visitdates(ii,:);
qaregstruct.visitdatenums = vdatenums(ii,:);
qaregstruct.acqsites = acqsites(ii,:);
qaregstruct.acqsitenos = acqsitenos(ii,:);
qaregstruct.visitnos = visitnos(ii,:);
qaregstruct.runnos = runnos(ii,:);
qaregstruct.measnames = measnames;

return;
