function [qastruct] = LoadQASum(qasumfile)
% qastruct = LoadQASum(qasumfile)
%
% Loads in the file created with fips-qa-sum. Eg:
%   fips-qa-sum --sum qa.dat --flac fb3-wm-sm5
%
% qastruct has the following members:
%  qatable 
%  birnids - N-by-1 string array of birnids 
%  visitdates - N-by-1 string array of  YYYY-MM-DD
%  visitdatenums - N-by-1 array of the visit date 
%     expressed as the number of days since Jan 1 2007.
%  acqsites - N-by-1 string array of acquisition sites
%  acqsitenos - N-by-1 array of acquisition sites numbers
%  visitnos - N-by-1 vector of visit numbers
%  runnos - N-by-1 vector of run numbers
%  xmldirs - N-by-1 vector of data directories
%  measnames - 10-by-M string array of the 10 qatable measures
% 
% qatable is N-by-10
% 1.  nmask - number of voxels in FEAT mask (1)
% 2.  mean func (after scaling) (2)
% 3.  rstd - residual stddev (afer scaling) (3)
% 4.  fsnr (4)
% 5.  fwhmx (5)
% 6.  fwhmy (6)
% 7.  fwhmz (7)
% 8.  mean func (before scaling)  (8)
% 9.  fsnr2 - fsnr squared (9)
% 10. rvar - resisual variance (rstd squared, after scaling) (10)
% 11. outcount (if present)
% There are also fields by each of those names.
%
% $Id: LoadQASum.m,v 1.2 2007/10/09 21:11:15 dgreve Exp $

qastruct = [];

if(nargin ~= 1)
  fprintf('qastruct = LoadQASum(qasumfile)\n');
  return
end

measnames = '';
measnames = strvcat(measnames,'Number of Voxels in Brain Mask');
measnames = strvcat(measnames,'Mean Functional Intensity');
measnames = strvcat(measnames,'Mean Residual Error StdDev');
measnames = strvcat(measnames,'Mean FSNR');
measnames = strvcat(measnames,'FWHM-x (mm)');
measnames = strvcat(measnames,'FWHM-y (mm)');
measnames = strvcat(measnames,'FWHM-z (mm)');
measnames = strvcat(measnames,'Mean Functional Intensity (before scaling)');
measnames = strvcat(measnames,'Mean FSNR^2');
measnames = strvcat(measnames,'Mean Residual Error Variance (before scaling)');

fp = fopen(qasumfile,'r');
if(fp == -1)
  fprintf('ERROR: could not open %s\n',qasumfile);
  return;
end

birnids = [];
visitdates = [];
acqsites = [];
acqsitenos = [];
visitnos = [];
runnos = [];
qatable = [];
xmldirs = [];
vdatenums = [];

% Serial date number of Jan 1 2007 to use as a baseline
vdatenum0 = datenum(2007,1,1);

tic;
nrows = 0;
while(1)
  nrows = nrows + 1;

  fscanf(fp,'%s',1); % past nth
  [bid   count] = fscanf(fp,'%s',1);
  if(count ~= 1) break; end
  [vyear  count]   = fscanf(fp,'%d',1);
  [vmonth count]   = fscanf(fp,'%d',1);
  [vday   count]   = fscanf(fp,'%d',1);
  [acqsite count] = fscanf(fp,'%s',1);
  [vno count]     = fscanf(fp,'%d',1);
  [rno count]     = fscanf(fp,'%d',1);
  [qaline count]  = fscanf(fp,'%f',10);
  %[qaline count]  = fscanf(fp,'%f',11);
  %[xmldir  count] = fscanf(fp,'%s',1);
  
  birnids = strvcat(birnids,bid);
  vdate = sprintf('%4d-%02d-%02d',vyear,vmonth,vday);
  visitdates = strvcat(visitdates,vdate);
  acqsites   = strvcat(acqsites,acqsite);
  %acqsiteno = sscanf(acqsite,'%d');
  %acqsitenos = [acqsitenos; acqsiteno];
  visitnos = [visitnos; vno];
  runnos = [runnos; rno];
  %xmldirs = strvcat(xmldirs,xmldir);
  
  qatable = [qatable; qaline'];

  % This computes the serial date number for the visit
  % relative to Jan 1 2007
  %ind = find(vdate == '-');
  %vdate(ind) = ' ';
  %ymd = sscanf(vdate,'%d %d %d');

  ymd = [vyear vmonth vday];
  vdatenum = datenum(ymd(1),ymd(2),ymd(3)) - vdatenum0;
  vdatenums = [vdatenums; vdatenum];

end
%fprintf('%4d %g\n',nrows,toc);

fclose(fp);

% Sort by Acq Date
[a ii] = sort(vdatenums);

qastruct.qatable = qatable(ii,:);
qastruct.birnids = birnids(ii,:);
qastruct.visitdates = visitdates(ii,:);
qastruct.visitdatenums = vdatenums(ii,:);
qastruct.acqsites = acqsites(ii,:);
%qastruct.acqsitenos = acqsitenos(ii,:);
qastruct.visitnos = visitnos(ii,:);
qastruct.runnos = runnos(ii,:);
%qastruct.xmldirs = xmldirs(ii,:);
qastruct.measnames = measnames;
qastruct.nmask = qastruct.qatable(:,1);
qastruct.mean_func = qastruct.qatable(:,2);
qastruct.rstd = qastruct.qatable(:,3);
qastruct.fsnr = qastruct.qatable(:,4);
qastruct.fwhmx = qastruct.qatable(:,5);
qastruct.fwhmy = qastruct.qatable(:,6);
qastruct.fwhmz = qastruct.qatable(:,7);
qastruct.mean_func_pre = qastruct.qatable(:,8);
qastruct.fsnr2 = qastruct.qatable(:,9);
qastruct.rvar = qastruct.qatable(:,10);
if(size(qastruct.qatable,2) == 11)
  qastruct.outcount = qastruct.qatable(:,11);
else
  qastruct.outcount = [];
end


return;
