function QAPlotSort(varargin)
% QAPlotSort(qa)
%  qa is either a qafile or a qastruct
%
% Displays the last clicked point and current mouse point
% in the current figure. If there is an image in the window,
% the value at the point is also printed out.
%
% To use, create the figure (eg, using plot or image), then, with that
% figure as the current figure, run showfigxy;
%
% Run showfigxy('help') for key press commands or hit 'h'
% when the cursor is in the figure to print help to terminal.
%
% To change the crosshair color, run 
%    showfigxy('crosshaircolor','newcolor') 
% where newcolor is a single letter color spec accepted by plot.
%
% Note: this will take over keyboard and mousing callbacks!
%
%
%

if(nargin == 0)  
  fprintf('QAPlotSort(qa)\n');
  return;
end

event = varargin{1};

if(strcmp(event,'help'))
  printhelp;
  return;
end

if(~strcmp(event,'init'))
  hfig = gcf;
end

%--------------------------------------------------------------%
switch(event)
 
  case 'init'
   hfig = figure;
   set(hfig,'color',[1 1 1]);
   set(hfig,'KeyPressFcn','QAPlotSort(''kbd'')');
   set(hfig,'WindowButtonDownFcn','QAPlotSort(''wbd'')');
   set(hfig,'WindowButtonMotionFcn','QAPlotSort(''wbm'')');
   ud = get(hfig,'UserData');
   ud.showfigxy = 1;
   ud.curxytxt = uicontrol('Style', 'text','Position',  [1 1 250 30]);
   ud.mvxytxt = uicontrol('Style', 'text','Position',  [260 1 250 30]);
   ax = axis;
   ud.curxy = [(ax(1)+ax(2))/2 (ax(3)+ax(4))/2];
   ud.mvxy  = [(ax(1)+ax(2))/2 (ax(3)+ax(4))/2];
   ud.hcursor1 = [];
   ud.hcursor2 = [];
   ud.zoom  = 0;
   ud.xzoom = 0;
   ud.yzoom = 0;
   ud.plotbydate = 0;
   ud.usecrosshair = 1;
   ud.crosshaircolor = 'g';
   ud = drawcrosshair(ud);
   qa = varargin{2};
   if(ischar(qa))
     qasumfile = qa;
     ud.qas = LoadQASum(qasumfile)
     if(isempty(ud.qas)) return; end
   else
     ud.qas = qa;
   end
   ud.qas.nthmeas = 1;
   if(nargin == 3) ud.qas.nthmeas = varargin{3}; end
   ud = qasort(ud);
   ud = redraw(ud);
   set(hfig,'UserData',ud);
   
 case 'crosshaircolor'
  if(nargin == 2) 
    ud = get(hfig,'UserData');
    if(~isshowfigxy(ud)) return; end
    ud.crosshaircolor = varargin{2};
    ud = drawcrosshair(ud); 
    set(hfig,'UserData',ud);
  end
  
 case 'wbd';
  ud = get(hfig,'UserData');
  if(~isshowfigxy(ud)) return; end
  xyz = get(gca,'CurrentPoint');
  x = round(xyz(1,1));
  y = xyz(1,2);  
  ax = axis;
  if(x < ax(1) | x > ax(2) | y < ax(3) | y > ax(4)) return; end
  %fprintf('x = %g, y = %g\n',x,y);
  meas = ud.qas.qatable(:,ud.qas.nthmeas);
  e = 10e10*ones(1,4);
  ind = zeros(1,4);
  hit = 0;
  if(x > 0 & x <= ud.qas.nDuke) 
    ind(1) = ud.qas.indDuke(x);
    e(1) = meas(ind(1));
    hit = 1;
  end
  if(x > 0 & x <= ud.qas.nBWH) 
    ind(2) = ud.qas.indBWH(x);
    e(2) = meas(ind(2));
    hit = 1;
  end
  if(x > 0 & x <= ud.qas.nMGH) 
    ind(3) = ud.qas.indMGH(x);
    e(3) = meas(ind(3));
    hit = 1;
  end
  if(x > 0 & x <= ud.qas.nYale) 
    ind(4) = ud.qas.indYale(x);
    e(4) = meas(ind(4));
    hit = 1;
  end
  if(hit)
    [m nthsite]=min(abs(e-y));
    bid = deblank(ud.qas.birnids(ind(nthsite),:));
    acqsite = deblank(ud.qas.acqsites(ind(nthsite),:));
    vdate = deblank(ud.qas.visitdates(ind(nthsite),:));
    vno = ud.qas.visitnos(ind(nthsite));
    rno = ud.qas.runnos(ind(nthsite));
    qaline = ud.qas.qatable(ind(nthsite),:);
    xmldir = deblank(ud.qas.xmldirs(ind(nthsite),:));
    fprintf('\n');
    fprintf('%s %s %s %d %d %g\n',bid,acqsite,vdate,vno,rno,e(nthsite));
    fprintf('nmask    %d\n',qaline(1));
    fprintf('meanfunc %g\n',qaline(2));
    fprintf('resstd   %g\n',qaline(3));
    fprintf('fsnr     %g\n',qaline(4));
    fprintf('fwhm     %6.2f %6.2f %6.2f \n',qaline(5),qaline(6),qaline(7));
    fprintf('%s \n',xmldir);
    set(hfig,'UserData',ud);
    ud.curxy = [x e(nthsite)];
  else
    ud.curxy = [x y];
  end
  set(hfig,'UserData',ud);
  setxystring(hfig,'cur');
  ud = drawcrosshair(ud);
  set(hfig,'UserData',ud);
  %keyboard
  
 case 'wbm';
  ud = get(hfig,'UserData');
  if(~isshowfigxy(ud)) return; end
  xyz = get(gca,'CurrentPoint');
  x = xyz(1,1);
  y = xyz(1,2);  
  ax = axis;
  if(x < ax(1) | x > ax(2) | y < ax(3) | y > ax(4)) return; end
  ud.mvxy = [x y];
  set(hfig,'UserData',ud);
  setxystring(hfig,'mv');
  %fprintf('x = %g, y = %g\n',x,y);
  
 case 'kbd';

  ud = get(hfig,'UserData');
  if(~isshowfigxy(ud)) return; end
  c = get(hfig,'CurrentCharacter'); 
  %fprintf('c = %s (%d)\n',c,c);

  switch(c)
   
   case 'c', 
    ud.usecrosshair = ~ud.usecrosshair;
    ud = drawcrosshair(ud);
     
   case 'h', 
    printhelp;
   
   case 'd', 
    ud.plotbydate = ~ud.plotbydate;
    ud = redraw(ud);
    set(hfig,'UserData',ud);
     
   case 'm', 
    ud.qas.nthmeas = ud.qas.nthmeas+1;
    if(ud.qas.nthmeas > 10) ud.qas.nthmeas = 1; end
    ud = qasort(ud);
    ud = redraw(ud);
    set(hfig,'UserData',ud);
     
   case 'x', 
    ud.xzoom = ~ud.xzoom ;
    if(ud.xzoom) zoom xon;
    else         zoom off;
    end
   
   case 'y', zoom;
    ud.yzoom = ~ud.yzoom ;
    if(ud.yzoom) zoom yon;
    else         zoom off;
    end
   
   case 'z', zoom;
    ud.zoom = ~ud.zoom ;
    if(ud.zoom)   
      ud.xzoom = 1; 
      ud.yzoom = 1; 
      zoom on;
    else
      ud.xzoom = 0; 
      ud.yzoom = 0; 
      zoom off;
    end
  
  end
 
  set(hfig,'UserData',ud);
end

return;  
%---------------------------------------------------------%

%---------------------------------------------------------%
function setxystring(hfig,type)
  img = isimage;
  ud = get(hfig,'UserData');
  if(strcmp(type,'cur'))
    x = ud.curxy(1); 
    y = ud.curxy(2);
    htxt = ud.curxytxt;
  else
    x = ud.mvxy(1); 
    y = ud.mvxy(2);
    htxt = ud.mvxytxt;
  end
  r = round(y);
  c = round(x);

  xystring = sprintf('x = %g, y = %g ',x,y);
  if(~isempty(img) & r > 0 & r <= size(img,1) & c > 0 & c <= size(img,2) )
    v = img(round(y),round(x));
    xystring = sprintf('%s, v = %g',xystring,v);
  end

  set(htxt,'string',xystring);
return;  

%---------------------------------------------------------%
function img = isimage
  % Image if the current axis has an image in it.
  % Bug: not correct for bar graphs
  img = [];
  
  chlist = get(gca,'children');
  nc = length(chlist);
  for n = 1:nc
    c = chlist(n);
    s = get(c);
    if(isfield(s,'CData')) 
      img = s.CData; 
      return; 
    end
  end

return;

%---------------------------------------------------------%
function ud = drawcrosshair(ud)
  if(ud.usecrosshair)
    if(~isempty(ud.hcursor1) & ishandle(ud.hcursor1))
      delete(ud.hcursor1);
      delete(ud.hcursor2);
    end
    a = axis;
    x = ud.curxy(1);
    y = ud.curxy(2);
    hold on;
    ud.hcursor1 = plot([a(1) a(2)],[y y],ud.crosshaircolor);
    ud.hcursor2 = plot([x x],[a(3) a(4)],ud.crosshaircolor);
    hold off;
  else
    if(~isempty(ud.hcursor1) & ishandle(ud.hcursor1))
      delete(ud.hcursor1);
      delete(ud.hcursor2);
    end
  end
   
return;

%---------------------------------------------------------%
function r = isshowfigxy(ud)
  if(~isfield(ud,'showfigxy')) r = 0;
  else r = 1;
  end
return;

%---------------------------------------------------------%
function printhelp
  fprintf('\n');
  help showfigxy;
  fprintf('c - toggle crosshair\n');
  fprintf('d - toggle double buffering\n');
  fprintf('h - print help\n');
  fprintf('x - toggle x zoom\n');
  fprintf('y - toggle y zoom\n');
  fprintf('z - toggle zoom\n');
  fprintf('\n');

return;

%---------------------------------------------------------%
function ud = qasort(ud)
  [ud.qas.qatable ind] = sortrows(ud.qas.qatable,ud.qas.nthmeas);
  ud.qas.birnids = ud.qas.birnids(ind,:);
  ud.qas.visitdates = ud.qas.visitdates(ind,:);
  ud.qas.acqsites = ud.qas.acqsites(ind,:);
  ud.qas.visitnos = ud.qas.visitnos(ind,:);
  ud.qas.runnos   = ud.qas.runnos(ind,:);
  ud.qas.xmldirs  = ud.qas.xmldirs(ind,:);
  ud.qas.indDuke = strmatch('0003',ud.qas.acqsites); 
  ud.qas.indBWH  = strmatch('0005',ud.qas.acqsites); 
  ud.qas.indMGH  = strmatch('0006',ud.qas.acqsites); 
  ud.qas.indYale = strmatch('0018',ud.qas.acqsites); 
  ud.qas.nDuke = length(ud.qas.indDuke);
  ud.qas.nBWH = length(ud.qas.indBWH);
  ud.qas.nMGH = length(ud.qas.indMGH);
  ud.qas.nYale = length(ud.qas.indYale);
return;

%---------------------------------------------------------%
function ud = redraw(ud)
  meas = ud.qas.qatable(:,ud.qas.nthmeas);
  if(ud.plotbydate)
    nnDuke = ud.qas.visitdatenums(ud.qas.indDuke);
    nnBWH = ud.qas.visitdatenums(ud.qas.indBWH);
    nnMGH = ud.qas.visitdatenums(ud.qas.indMGH);
    nnYale = ud.qas.visitdatenums(ud.qas.indYale);
    xlab = 'Serial Date Number';
    pc = '.';
  else
    nnDuke = 1:ud.qas.nDuke;
    nnBWH = 1:ud.qas.nBWH;
    nnMGH = 1:ud.qas.nMGH;
    nnYale = 1:ud.qas.nYale;
    xlab = 'Sort Number';
    pc = '-';
  end
  
  plot(nnDuke,(meas(ud.qas.indDuke)),pc,...
       nnBWH,(meas(ud.qas.indBWH)),pc,...
       nnMGH,(meas(ud.qas.indMGH)),pc,...
       nnYale,(meas(ud.qas.indYale)),pc );

  xlabel(xlab);
  title(deblank(ud.qas.measnames(ud.qas.nthmeas,:)));
  legend('Duke','BWH','MGH','Yale',4);
  set(gcf,'color',[1 1 1]);
return;
